
From: Jeff Dike <jdike@addtoit.com>

The patch below fixes a few UML-specific bugs not related to the rest of the
kernel
	a bogus error return and some formatting in the fork code
	correct calculation of task.thread.kernel_stack
	remove a bogus panic
	a couple of fixes to allow UML to boot in the presence of exec-shield


Signed-off-by: Andrew Morton <akpm@osdl.org>
---

 25-akpm/arch/um/kernel/process.c         |   10 +++++++---
 25-akpm/arch/um/kernel/process_kern.c    |    2 +-
 25-akpm/arch/um/kernel/trap_kern.c       |    2 --
 25-akpm/arch/um/kernel/tt/mem.c          |    2 +-
 25-akpm/arch/um/kernel/tt/process_kern.c |    2 +-
 5 files changed, 10 insertions(+), 8 deletions(-)

diff -puN arch/um/kernel/process.c~uml-fixes arch/um/kernel/process.c
--- 25/arch/um/kernel/process.c~uml-fixes	2004-08-15 15:08:51.941665832 -0700
+++ 25-akpm/arch/um/kernel/process.c	2004-08-15 15:08:51.950664464 -0700
@@ -124,10 +124,14 @@ int start_fork_tramp(void *thread_arg, u
 
 	/* Start the process and wait for it to kill itself */
 	new_pid = clone(outer_tramp, (void *) sp, clone_flags, &arg);
-	if(new_pid < 0) return(-errno);
+	if(new_pid < 0)
+		return(new_pid);
+
 	CATCH_EINTR(err = waitpid(new_pid, &status, 0));
-	if(err < 0) panic("Waiting for outer trampoline failed - errno = %d", 
-			  errno);
+	if(err < 0)
+		panic("Waiting for outer trampoline failed - errno = %d",
+		      errno);
+
 	if(!WIFSIGNALED(status) || (WTERMSIG(status) != SIGKILL))
 		panic("outer trampoline didn't exit with SIGKILL, "
 		      "status = %d", status);
diff -puN arch/um/kernel/process_kern.c~uml-fixes arch/um/kernel/process_kern.c
--- 25/arch/um/kernel/process_kern.c~uml-fixes	2004-08-15 15:08:51.942665680 -0700
+++ 25-akpm/arch/um/kernel/process_kern.c	2004-08-15 15:08:51.951664312 -0700
@@ -167,7 +167,7 @@ int copy_thread(int nr, unsigned long cl
 {
 	p->thread = (struct thread_struct) INIT_THREAD;
 	p->thread.kernel_stack = 
-		(unsigned long) p->thread_info + THREAD_SIZE;
+		(unsigned long) p->thread_info + 2 * PAGE_SIZE;
 	return(CHOOSE_MODE_PROC(copy_thread_tt, copy_thread_skas, nr, 
 				clone_flags, sp, stack_top, p, regs));
 }
diff -puN arch/um/kernel/trap_kern.c~uml-fixes arch/um/kernel/trap_kern.c
--- 25/arch/um/kernel/trap_kern.c~uml-fixes	2004-08-15 15:08:51.944665376 -0700
+++ 25-akpm/arch/um/kernel/trap_kern.c	2004-08-15 15:08:51.951664312 -0700
@@ -54,8 +54,6 @@ int handle_page_fault(unsigned long addr
 	if(is_write && !(vma->vm_flags & VM_WRITE)) 
 		goto out;
 	page = address & PAGE_MASK;
-	if(address < (unsigned long) current_thread + 1024 && !is_user)
-		panic("Kernel stack overflow");
 	pgd = pgd_offset(mm, page);
 	pmd = pmd_offset(pgd, page);
 	do {
diff -puN arch/um/kernel/tt/mem.c~uml-fixes arch/um/kernel/tt/mem.c
--- 25/arch/um/kernel/tt/mem.c~uml-fixes	2004-08-15 15:08:51.945665224 -0700
+++ 25-akpm/arch/um/kernel/tt/mem.c	2004-08-15 15:08:51.951664312 -0700
@@ -18,7 +18,7 @@ void before_mem_tt(unsigned long brk_sta
 	if(!jail || debug)
 		remap_data(UML_ROUND_DOWN(&_stext), UML_ROUND_UP(&_etext), 1);
 	remap_data(UML_ROUND_DOWN(&_sdata), UML_ROUND_UP(&_edata), 1);
-	remap_data(UML_ROUND_DOWN(&__bss_start), UML_ROUND_UP(brk_start), 1);
+	remap_data(UML_ROUND_DOWN(&__bss_start), UML_ROUND_UP(&_end), 1);
 }
 
 #ifdef CONFIG_HOST_2G_2G
diff -puN arch/um/kernel/tt/process_kern.c~uml-fixes arch/um/kernel/tt/process_kern.c
--- 25/arch/um/kernel/tt/process_kern.c~uml-fixes	2004-08-15 15:08:51.947664920 -0700
+++ 25-akpm/arch/um/kernel/tt/process_kern.c	2004-08-15 15:08:51.952664160 -0700
@@ -412,7 +412,7 @@ static void mprotect_kernel_mem(int w)
 	protect_memory(start, end - start, 1, w, 1, 1);
 
 	start = (unsigned long) UML_ROUND_DOWN(&__bss_start);
-	end = (unsigned long) UML_ROUND_UP(brk_start);
+	end = (unsigned long) UML_ROUND_UP(&_end);
 	protect_memory(start, end - start, 1, w, 1, 1);
 
 	mprotect_kernel_vm(w);
_
